#include <bits/stdc++.h>
using namespace std;

using vi = vector<int>;

constexpr int D = 1000010;

// Point query / range update over [0..N)
template <int N>
struct SegmentTree {
	vi tree;
	SegmentTree() {
		int SZ = 1;
		while ((SZ >> 1) < N) SZ <<= 1;
		tree.assign(SZ, 0LL);
	}
	void update(int l, int r, int u, int i = 1, int _l = 0, int _r = N - 1) {
		if (_r < l || r < _l) return;
		if (l <= _l && _r <= r) {
			tree[i] += u;
			return;
		} else {
			_push(i);
			int _m = (_l + _r) >> 1;
			update(l, r, u, (i<<1), _l, _m);
			update(l, r, u, (i<<1)|1, _m+1, _r);
		}
	}
	int query(int p, int i = 1, int _l = 0, int _r = N - 1) {
		if (_l == _r)
			return tree[i];
		_push(i);
		int _m = (_l + _r) >> 1;
		if (p <= _m)
			return query(p, (i<<1), _l, _m);
		else
			return query(p, (i<<1)|1, _m+1, _r);
	}
private:
	void _push(int i) {
		tree[(i<<1)] += tree[i];
		tree[(i<<1)|1] += tree[i];
		tree[i] = 0;
	}
};

// Jumping to a position in [l, r] costs `cost`.
struct interval { int l, r, cost; };
bool interval_compare(const interval &a, const interval &b) {
	return a.l < b.l;
}

void update_around(int p, SegmentTree<D> &ft, vector<interval> &ivs, int m) {
	for (const interval &iv : ivs)
		ft.update(p + iv.l, p + iv.r, m * iv.cost);
}
void add_frog(int p, SegmentTree<D> &fteven, SegmentTree<D> &ftodd,
		vector<interval> &even, vector<interval> &odd, int m = 1) {
	update_around(p, fteven, (p&1) ? odd : even, m);
	update_around(p, ftodd, (p&1) ? even : odd, m);
}
void remove_frog(int p, SegmentTree<D> &fteven, SegmentTree<D> &ftodd,
		vector<interval> &even, vector<interval> &odd) {
	add_frog(p, fteven, ftodd, even, odd, -1);
}

int main() {
	ios::sync_with_stdio(false);
	cin.tie(NULL);

	// Preprocessing: Get costs to aint even/odd positions at a large enough
	// distance.
	vector<interval> even, odd;
	{
		// Initialize with [0, 0] having cost 0.
		even.push_back(interval{0, 0, 0});
		odd.push_back(interval{0, 0, 0});
		int jumps = 0, d = 0;
		while (min(even.back().r, odd.back().r) < D) {
			// Jump!
			jumps++, d += jumps;

			// Add a new interval. We can reach everything upto distance `d`
			// with `jumps` jumps, so long as the parity of the position and
			// `d` are the same.
			vector<interval> &type = (d&1) ? odd :even;
			type.push_back(interval{type.back().r + 1, d, jumps});
		}
		// Mirror in both directions. Start at 1 to avoid [0, 0].
		size_t E = even.size(), O = odd.size();
		for (size_t i = 1; i < E; ++i)
			even.push_back(interval{-even[i].r, -even[i].l, even[i].cost});
		for (size_t i = 1; i < O; ++i)
			odd.push_back(interval{-odd[i].r, -odd[i].l, odd[i].cost});
		sort(even.begin(), even.end(), interval_compare);
		sort(odd.begin(), odd.end(), interval_compare);
	}

	// Range updates / point queries.
	SegmentTree<D> fteven, ftodd;

	int n, t;
	cin >> n >> t;
	while (n--) {
		int p;
		cin >> p;
		add_frog(p, fteven, ftodd, even, odd);
	}

	int C;
	cin >> C;
	while (C--) {
		char q; int a;
		cin >> q >> a;
		if (q == '+')
			add_frog(a, fteven, ftodd, even, odd);
		else if (q == '-')
			remove_frog(a, fteven, ftodd, even, odd);
		else // q == 't'
			t = a;

		cout << ((t&1) ? ftodd.query(t) : fteven.query(t)) << '\n';
	}
}
